/*
  LcdUi.cpp

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
 */

#include <LiquidCrystal.h>
#include "LcdUi.h"

/*
  Constructor

  parameters :
  width                     number of characters
  height                    number of display rows
  rs, rw, enable, d0 - d7   see LiquidCrystal library parameter
*/
LcdUi::LcdUi(uint8_t width,uint8_t height,uint8_t rs,uint8_t rw,uint8_t enable,uint8_t d0,uint8_t d1,uint8_t d2, uint8_t d3, uint8_t d4,uint8_t d5, uint8_t d6, uint8_t d7):
  m_width(width),m_height(height),m_lcd(rs,rw,enable,d0,d1,d2,d3,d4,d5,d6,d7),m_edit(false) {
    m_lcd.begin(width,height);
}
LcdUi::LcdUi(uint8_t width,uint8_t height,uint8_t rs,uint8_t enable,uint8_t d0,uint8_t d1,uint8_t d2, uint8_t d3, uint8_t d4,uint8_t d5, uint8_t d6, uint8_t d7):
  m_width(width),m_height(height),m_lcd(rs,enable,d0,d1,d2,d3,d4,d5,d6,d7),m_edit(false) {
    m_lcd.begin(width,height);
}
LcdUi::LcdUi(uint8_t width,uint8_t height,uint8_t rs,uint8_t enable,uint8_t d4,uint8_t d5, uint8_t d6, uint8_t d7):
  m_width(width),m_height(height),m_lcd(rs,enable,d4,d5,d6,d7),m_edit(false) {
    m_lcd.begin(width,height);
}

/*
  Changed.
  Check whether specified field is being edited.

  parameters :
  menu        menu number
  field       field number
  return      true if being edited
              false otherwise
*/
uint8_t LcdUi::changed(uint8_t menu,uint8_t field) {
  if (m_edit && !(m_edit = LcdUi::edit(menu,field))) return true;
  m_edit = LcdUi::edit(menu,field);
  return false;
}

/*
  Edit.
  Check whether specified field is being edited.

  parameters :
  menu        menu number
  field       field number
  return      true if being edited
              false otherwise
*/
uint8_t LcdUi::edit(uint8_t menu,uint8_t field) {
  return m_menus.id == menu && m_menus.menu.fields.id == field && m_menus.menu.fields.field.status == LCDEDIT_CHANGED;
}

/*
  Init

  parameters :
  DEF         pointer to menu definition in PROGMEM
  data        pointer to string data
  pin         analog input pin number
  debounce    debounce time (ms)
  delay       keyboard repeat delay (ms)
  repeat      keyboard repeat rate (char/s)
  exit        inactivity time out (s)
*/
void LcdUi::init(PGM_VOID_P DEF,char* data,uint8_t pin,uint16_t debounce,uint16_t delay,uint8_t repeat,uint8_t exit) {
  m_menus.DEF = DEF;
  m_menus.data = data;
  LcdMenusInit(&m_menus,true);
  LcdUi::screen();
  LcdKeyInit(pin,debounce,delay,repeat,exit);
}

/*
  Screen.
  Output menu (from PROGMEM/data) to screen.
*/
void LcdUi::screen() {
  LcdMenuGet(&m_menus.menu,true);
  for (uint8_t i = 0;i < m_height;i++) {
    m_lcd.setCursor(0,i);
    for (uint8_t j = 0;j < m_width;j++) m_lcd.write(LcdMenuGet(&m_menus.menu,false));
  }
  if (m_menus.menu.fields.status == LCDEDIT_EDIT || m_menus.menu.fields.status == LCDEDIT_CHANGED) {
    m_lcd.blink();
    m_lcd.setCursor(m_menus.menu.cursor % m_width,m_menus.menu.cursor / m_width);
  } else m_lcd.noBlink();
}

/*
  Refresh.
  Call periodically to process key presses/refresh.

  parameters :
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdUi::refresh() {
  enum LCDKEY key = LcdKeyPress();
  if (key == LCDKEY_NONE) return LCDKEY_NONE;
  key = LcdMenusEdit(key,&m_menus);
/*
  switch(key) {
    case LCDKEY_NONE:
      break;
    default:
      key = LCDKEY_NONE;
    break;
    }
  }
*/
  LcdUi::screen();
  return key;
}
