/*
  UtLcdMenu.ino
  Unit tests.

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "awl.h"
#include "LcdUi.h"

TestSuite suite;

int8_t MENU0[] PROGMEM = {1,2,3,4};
char SCREEN0[] PROGMEM = "----- base -----"  //base menu (dummy left, up and down destinations)
                         "left up/dn right";
int8_t MENU1[] PROGMEM = {0,0,0,-1};
char SCREEN1[] PROGMEM = "----- left -----"  //left menu
                         "~          right";
char TYPE10[] PROGMEM = "ccec";  //field 1 character = 1
int8_t MENU2[] PROGMEM = {0,0,-2,0};
char SCREEN2[] PROGMEM = "-----  up  -----"  //up menu
                         "~       dn     ~";
char TYPE20[] PROGMEM = "cfhf";  //field 1 character = 1
char TYPE21[] PROGMEM = "chjh";  //field 2 character = 1
int8_t MENU3[] PROGMEM = {0,-3,0,0};
char SCREEN3[] PROGMEM = "----- down -----"  //down menu
                         "     up         ";
int8_t MENU4[] PROGMEM = {-4,0,0,1};
char SCREEN4[] PROGMEM = "----- right ----"  //right menu
                         "left          IO";
int8_t MENU5[] PROGMEM = {-1,0,0,0};
char SCREEN5[] PROGMEM = "~~~~~~~~ ~~~ ~ ~"  //IO menu
                         "~~~~~~~~ ~~~ ~~~";
char TYPE50[] PROGMEM = "d01/01/10";    //field 1 date = 8
char TYPE51[] PROGMEM = "i012345012";   //field 2 integer = 3
char TYPE52[] PROGMEM = "caca";         //field 3 character = 1
char TYPE53[] PROGMEM = "cdfd";         //field 4 character = 1
char TYPE54[] PROGMEM = "t08:00:00";    //field 5 time = 8
char TYPE55[] PROGMEM = "saaafghaaa";   //field 5 string = 3
char TYPE56[] PROGMEM = "o003---";      //field 6 output = 3

PGM_VOID_P TYPE1[] PROGMEM = {0,TYPE10,0};
PGM_VOID_P TYPE2[] PROGMEM = {0,TYPE20,TYPE21,0};
PGM_VOID_P TYPE5[] PROGMEM = {0,TYPE50,TYPE51,TYPE52,TYPE53,TYPE54,TYPE55,TYPE56,0};

PGM_VOID_P MENU[] PROGMEM = {
  0,0,
  MENU0,0,
  MENU1,TYPE1,
  MENU2,TYPE2,
  MENU3,0,
  MENU4,0,
  MENU5,TYPE5,
  0,0
};

LcdMenus mn;
char str_i[40];
char str_o[33];

void setup() {}

test(get_id) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  assertEquals(0,mn.id);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  assertEquals(4,mn.id);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  assertEquals(5,mn.id);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  assertEquals(4,mn.id);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  assertEquals(0,mn.id);
}

test(get_menu0) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  LcdMenuGet(&mn.menu,true);
  for (int i = 0;i < 32;i++) str_o[i] = LcdMenuGet(&mn.menu,false);
  str_o[32] = 0;
  assertEquals("----- base -----left up/dn right",str_o);
}

test(get_menu1) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  LcdMenuGet(&mn.menu,true);
  for (int i = 0;i < 32;i++) str_o[i] = LcdMenuGet(&mn.menu,false);
  str_o[32] = 0;
  assertEquals("----- left -----c          right",str_o);
}

test(get_menu2) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_UP;
  LcdMenusEdit(key,&mn);
  LcdMenuGet(&mn.menu,true);
  for (int i = 0;i < 32;i++) str_o[i] = LcdMenuGet(&mn.menu,false);
  str_o[32] = 0;
  assertEquals("-----  up  -----f       dn     h",str_o);
}

test(get_menu5) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  LcdMenuGet(&mn.menu,true);
  for (int i = 0;i < 32;i++) str_o[i] = LcdMenuGet(&mn.menu,false);
  str_o[32] = 0;
  assertEquals("01/01/10 012 a d08:00:00 aaa ---",str_o);
}

test(init) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  assertEquals(30,mn.length);
  assertEquals(1,mn.menu.left);
  assertEquals(2,mn.menu.up);
  assertEquals(3,mn.menu.down);
  assertEquals(4,mn.menu.right);
}

test(navigate_nok) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  assertEquals(-1,mn.menu.right);
  key = LCDKEY_RIGHT;
  LcdMenusEdit(key,&mn);
  assertEquals(4,mn.menu.right);
}

test(navigate_ok) {
  mn.DEF = MENU;
  mn.data = str_i;
  LcdMenusInit(&mn,true);
  LCDKEY key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  assertEquals(4,mn.menu.right);
  key = LCDKEY_SELECT;
  LcdMenusEdit(key,&mn);
  key = LCDKEY_LEFT;
  LcdMenusEdit(key,&mn);
  assertEquals(-1,mn.menu.right);
}

void loop() {
  suite.run();
}
