/*
  UtString.ino
  Unit tests.

  release : 0.0.1 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "awl.h"

TestSuite suite;

PROGMEM prog_char DATA[] =  "1234"                // +0  integer
                            "34"                  // +4  byte 1
                            "1234"                // +6  byte 2
                            "04/05/06"            // +10 date
                            "01:02:03 04/05/06";  // +18 time

void setup() {}

test(compare) {
  assertEquals(true,as_cmp("0123","0123456"));
  assertEquals(false,as_cmp("0124","0123456"));
}

test(compare_length) {
  assertEquals(true,as_ncmp("0134","0123456",2));
  assertEquals(false,as_ncmp("0234","0123456",2));
}

test(convert_date_to_string) {
  char string[9];
  string[8] = 0;
  ad_t d = {11,2,26};
  as_dtos(&d,string);
  assertEquals("26/02/11",string);
}

test(convert_int_to_string_1) {
  char string[2];
  string[1] = 0;
  as_itos(1,string,1);
  assertEquals("1",string);
}

test(convert_int_to_string_4) {
  char string[5];
  string[4] = 0;
  as_itos(1234,string,4);
  assertEquals("1234",string);
}

test(convert_int_to_string_1_from_2) {
  char string[2];
  string[1] = 0;
  as_itos(12,string,1);
  assertEquals("2",string);
}

test(convert_int_to_string_2_in_4) {
  char string[5];
  string[4] = 0;
  as_itos(12,string,4);
  assertEquals("0012",string);
}

test(convert_string_to_byte_2) {
  char string[] = "34";
  assertEquals(34,as_stob(string,2));
}

test(convert_string_to_byte_2_P) {
  assertEquals(34,as_stob_P(DATA + 4,2));
}

test(convert_string_to_byte_2_from_4) {
  char string[] = "1234";
  assertEquals(12,as_stob(string,2));
}

test(convert_string_to_byte_2_from_4_P) {
  assertEquals(12,as_stob_P(DATA + 6,2));
}

test(convert_string_to_date) {
  char string[] = "04/05/06";
  ad_t d;
  as_stod(string,&d);
  assertEquals(4,d.day);
  assertEquals(5,d.month);
  assertEquals(6,d.year);
}

test(convert_string_to_date_P) {
  ad_t d;
  as_stod_P(DATA + 10,&d);
  assertEquals(4,d.day);
  assertEquals(5,d.month);
  assertEquals(6,d.year);
}

test(convert_string_to_int_2_from_4) {
  char string[] = "1234";
  assertEquals(12,  as_stoi(string,2));
}

test(convert_string_to_int_2_from_4_P) {
  assertEquals(12,as_stoi_P(DATA,2));
}

test(convert_string_to_int_4) {
  char string[] = "1234";
  assertEquals(1234,as_stoi(string,4));
}

test(convert_string_to_int_4_P) {
  assertEquals(1234,as_stoi_P(DATA,4));
}

test(convert_string_to_time) {
  char string[] = "01:02:03 04/05/06";
  at_t t;
  as_stot(string,&t);
  assertEquals(1,t.hour);
  assertEquals(2,t.minute);
  assertEquals(3,t.second);
}

test(convert_string_to_time_P) {
  at_t t;
  as_stot_P(DATA + 18,&t);
  assertEquals(1,t.hour);
  assertEquals(2,t.minute);
  assertEquals(3,t.second);
}

test(convert_time_to_string) {
  char string[9];
  string[8] = 0;
  at_t t = {13,32,55};
  as_ttos(&t,string);
  assertEquals("13:32:55",string);
}

test(copy) {
  char dummy[10];
  as_cpy("0123",dummy);
  assertEquals(true,as_cmp("0123",dummy));
}

test(find) {
  assertEquals(0,as_find("0123456789","01",2));
  assertEquals(4,as_find("0123456789","89",2));
  assertEquals(-1,as_find("0123456789","12",2));
}

void loop() {
  suite.run();
}
