/*
  lcdui.ino
  LCD user interface example.

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "awl.h"
#include "LcdUi.h"

int8_t MENU0[] PROGMEM = {1,2,3,4};
char SCREEN0[] PROGMEM = "----- base -----"  //base menu (dummy left, up and down destinations)
                         "left up/dn right";
int8_t MENU1[] PROGMEM = {0,0,0,-1};
char SCREEN1[] PROGMEM = "----- left -----"  //left menu
                         "           right";
int8_t MENU2[] PROGMEM = {0,0,-2,0};
char SCREEN2[] PROGMEM = "-----  up  -----"  //up menu
                         "        dn      ";
int8_t MENU3[] PROGMEM = {0,-3,0,0};
char SCREEN3[] PROGMEM = "----- down -----"  //down menu
                         "     up         ";
int8_t MENU4[] PROGMEM = {-4,0,0,1};
char SCREEN4[] PROGMEM = "----- right ----"  //right menu
                         "left          IO";
int8_t MENU5[] PROGMEM = {-1,0,0,1};
char SCREEN5[] PROGMEM = "~~~~~~~~ ~~~ ~ ~"  //IO menu
                         "~~~~~~~~ ~~~ ~~~";
char TYPE50[] PROGMEM = "d01/01/10";    //field 1 date, length = 8
char TYPE51[] PROGMEM = "i012345012";   //field 2 integer, length = 3
char TYPE52[] PROGMEM = "caca";         //field 3 character, length = 1
char TYPE53[] PROGMEM = "cdfd";         //field 4 character, length = 1
char TYPE54[] PROGMEM = "t08:00:00";    //field 5 time, length = 8
char TYPE55[] PROGMEM = "saaafghaaa";   //field 5 string, length = 3
char TYPE56[] PROGMEM = "o003---";      //field 6 output, length = 3

PGM_VOID_P TYPE5[] PROGMEM = {0,TYPE50,TYPE51,TYPE52,TYPE53,TYPE54,TYPE55,TYPE56,0}; // pointers to I/O fields description

int8_t MENU6[] PROGMEM = {-1,0,0,0};
char SCREEN6[] PROGMEM = "~ ~~~~~~~       "  //IO menu 2
                         "left            ";
char TYPE60[] PROGMEM = "CaApPa";                                           //field 1 character selector
char TYPE61[] PROGMEM = "S005       Start  Stop   Auto   ManueelAuto   ";   //field 2 string selector

PGM_VOID_P TYPE6[] PROGMEM = {0,TYPE60,TYPE61,0}; // pointers to I/O fields description

PGM_VOID_P MENU[] PROGMEM = { // pointers to menu structure and (pointers to) fields description
  0,0,
  MENU0,0,
  MENU1,0,
  MENU2,0,
  MENU3,0,
  MENU4,0,
  MENU5,TYPE5,
  MENU6,TYPE6,
  0,0
};

LcdUi lcd = LcdUi(16,2,8,9,4,5,6,7); //LCD keypad : LCD pinout

char str[] = "DD/HH/MM123ccHH:MM:SSsssoooCSSSSSSS"; // dummy string assignment
//            00000000001111111111222222222233333  character counter
//            01234567890123456789012345678901234

adt_t dt; // date and time structure

unsigned int counter = 0; // just a counter (number of loops / second)

void setup() {
  lcd.init(MENU,str,0,50,500,10,30); // initialise user interface with menu structure, fields data and key : analog pin, debounce time, delay time, repeat frequency, exit time
  Serial.begin(9600);
  adt_set(&dt,"19/02/13","08:00:00");
  as_stod(str,&dt.date); // update clock
  as_stot(&str[13],&dt.time); // update clock
}


void loop() {
  lcd.refresh(); // process user interface
  if (as_stob(str,2) != dt.date.day || as_stob(str,3) != dt.date.month || as_stob(str,6) != dt.date.year) { // user changed date ? (string position 0)
    as_stod(str,&dt.date); // update clock
    Serial.println(dt.date.day); // just a print instruction
  }
  if (as_stob(&str[13],2) != dt.time.hour || as_stob(&str[16],2) != dt.time.minute || as_stob(&str[19],2) != dt.time.second) { // user changed time ? (string position 13)
    as_stot(&str[13],&dt.time); // update clock
    Serial.println(dt.time.hour); // just a print instruction
  }
  if (adt_update(&dt)) { // at least one second lapsed ?
    as_dtos(&dt.date,str); // output clock date (string position 0)
    as_ttos(&dt.time,&str[13]); // output clock time (string position 13)
    as_itos(counter,&str[24],3); // output counter (string position 24)
    counter = 0;
    lcd.screen();
  }
  ++counter;
}

