/*
  LcdEdit.c

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdEdit.h"

enum LCDKEY LcdEditByte(uint8_t length,uint8_t lower,uint8_t upper,uint8_t *x,enum LCDKEY key,char *data) {
  uint8_t numberOld = awlstring_stob(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t number = awlstring_stob(data,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  awlstring_itos(number,data,length);
  return key;
}

enum LCDKEY LcdEditByte_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *data) {
  uint8_t numberOld = awlstring_stob(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t number = awlstring_stob(data,length);
  uint8_t lower = awlstring_stob_P(DEF + 1,length);
  uint8_t upper = awlstring_stob_P(DEF + 1 + length,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  awlstring_itos(number,data,length);
  return key;
}

enum LCDKEY LcdEditChar_P(PGM_P DEF,enum LCDKEY key,char *data) {
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_LEFT: case LCDKEY_RIGHT: case LCDKEY_ESC: break;
    case LCDKEY_DOWN:
      --*data;
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++*data;
      key = LCDKEY_NONE;
      break;
  }
  if (*data < pgm_read_byte(DEF + 1)) *data = pgm_read_byte(DEF + 1); // lower
  if (*data > pgm_read_byte(DEF + 2)) *data = pgm_read_byte(DEF + 2); // upper
  return key;
}

enum LCDKEY LcdEditCharSel_P(PGM_P DEF,enum LCDKEY key,char *data) {
  uint8_t index = 0;
  uint8_t sels = strlen_P(DEF + 1);
  while ((index < sels) && (*data != pgm_read_byte(DEF + index + 1))) ++index;
  switch (key) {
    case LCDKEY_DOWN:
      *data = index < sels ? index > 0 ? pgm_read_byte(DEF + index) : *data : pgm_read_byte(DEF + 1);
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      *data = index < sels ? index < sels - 1 ? pgm_read_byte(DEF + index + 2) : *data : pgm_read_byte(DEF + index);
      key = LCDKEY_NONE;
      break;
    case LCDKEY_NONE: case LCDKEY_SELECT:
      key = LCDKEY_NONE;
    case LCDKEY_LEFT: case LCDKEY_RIGHT: case LCDKEY_ESC:
    default:
      if (index >= sels) *data = pgm_read_byte(DEF + 1);
    break;
  }
  return key;
}

enum LCDKEY LcdEditFix_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *data) {
  uint16_t numberOld;
  uint8_t dec;
  awlstring_stof(data,length,&numberOld,&dec);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        if (*x == length - dec) *x -= 2;
        else --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        if (*x == length - dec - 2) *x += 2;
        else ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint16_t lower;
  uint16_t upper;
  uint16_t number;
  awlstring_stof_P(DEF + 1,length,&lower,&dec);
  awlstring_stof_P(DEF + length + 1,length,&upper,&dec);
  awlstring_stof(data,length,&number,&dec);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  awlstring_ftos(number,dec,data,length);
  return key;
}

enum LCDKEY LcdEditInt_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *data) {
  uint16_t numberOld = awlstring_stoi(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint16_t lower = awlstring_stoi_P(DEF + 1,length);
  uint16_t upper = awlstring_stoi_P(DEF + length + 1,length);
  uint16_t number = awlstring_stoi(data,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  awlstring_itos(number,data,length);
  return key;
}

enum LCDKEY LcdEditString_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *data) {
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t i;
  for (i = 0;i < length;i++) {
    if (data[i] < pgm_read_byte(DEF + 1 + i)) data[i] = pgm_read_byte(DEF + 1 + i); // lower
    if (data[i] > pgm_read_byte(DEF + 1 + length + i)) data[i] = pgm_read_byte(DEF + 1 + length + i); // upper
  }
  return key;
}

enum LCDKEY LcdEditStringSel_P(PGM_P DEF,uint8_t length,enum LCDKEY key,char *data) {
  uint8_t index = 0;
  uint8_t sels = awlstring_stob_P(DEF + 1,3);
  while ((index < sels) && strncmp_P(data,DEF + index * length + 4,length)) ++index;
  switch (key) {
    case LCDKEY_DOWN:
      if (index < sels) {
        if (index > 0) strncpy_P(data,DEF + (index - 1) * length + 4,length);
      } else
        strncpy_P(data,DEF + 4,length);
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      if (index < sels) {
        if (index < sels - 1) strncpy_P(data,DEF + (index + 1) * length + 4,length);
      } else
        strncpy_P(data,DEF + (index - 2) * length + 4,length);
      key = LCDKEY_NONE;
      break;
    case LCDKEY_NONE: case LCDKEY_SELECT:
      key = LCDKEY_NONE;
    case LCDKEY_LEFT: case LCDKEY_RIGHT: case LCDKEY_ESC:
    default:
      if (index >= sels) strncpy_P(data,DEF + 4,length);
      break;
  }
  return key;
}
