/*
  LcdEdit.h
  Process character under cursor.

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LcdEdit_h
#define LcdEdit_h

#include <Arduino.h>
#include "LcdKey.h"
#include "../../awl/utility/awlString.h"

enum LCDEDIT {LCDEDIT_NONE,LCDEDIT_INIT,LCDEDIT_EDIT,LCDEDIT_CHANGED};

/*
  Edit byte.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited byte
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditByte(uint8_t length,uint8_t lower,uint8_t upper,uint8_t *x,enum LCDKEY key,char *string);

/*
  Edit byte.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited byte
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditByte_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *string);

/*
  Edit character.

  parameter:
  DEF       pointer to input definition = clu : character-lower-upper
  key       key to process
  data      edited character
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditChar_P(PGM_P DEF,enum LCDKEY key,char *string);

/*
  Edit character selector.

  parameter:
  DEF       pointer to input definition = C12...n : character-selection 1, selection2,... selection n
  key       key to process
  data      edited character
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditCharSel_P(PGM_P DEF,enum LCDKEY key,char *string);

/*
  Edit fixed point.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited integer
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditFix_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *string);

/*
  Edit integer.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited integer
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditInt_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *string);

/*
  Edit string.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditString_P(PGM_P DEF,uint8_t length,uint8_t *x,enum LCDKEY key,char *string);

/*
  Edit string selector.

  parameter :
  DEF       pointer to input definition = Snnn12...n : string-number of selections-selection 1, selection2,... selection n
  key       key to process
  length    string length
  data      edited string
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdEditStringSel_P(PGM_P DEF,uint8_t length,enum LCDKEY key,char *string);

#endif

#ifdef __cplusplus
}
#endif
