/*
  LcdField.c

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdField.h"

enum LCDKEY LcdFieldEdit(enum LCDKEY key,struct LcdField *field) {
  switch (key) {
    case LCDKEY_NONE: break;
    case LCDKEY_SELECT:
      if (field->status == LCDEDIT_INIT) {
        field->status = LCDEDIT_EDIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_ESC:
      if (field->status == LCDEDIT_EDIT || field->status == LCDEDIT_CHANGED) {
        field->status = LCDEDIT_INIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_EXIT: if (field->status == LCDEDIT_CHANGED) field->status = LCDEDIT_INIT; break;
    case LCDKEY_UP: case LCDKEY_DOWN:
      if (field->status == LCDEDIT_EDIT) field->status = LCDEDIT_CHANGED;
    case LCDKEY_LEFT: case LCDKEY_RIGHT:
      if (field->status == LCDEDIT_EDIT || field->status == LCDEDIT_CHANGED) {
        switch (pgm_read_byte(field->DEF)) {
          case 'b': key = LcdEditByte_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 'c': key = LcdEditChar_P(field->DEF,key,field->data); break;
          case 'd': key = LcdEditTimeDate(&field->x,key,field->data); break;
          case 'f': key = LcdEditFix_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 'i': key = LcdEditInt_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 's': key = LcdEditString_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 't': key = LcdEditTimeTime(&field->x,key,field->data); break;
          case 'C': key = LcdEditCharSel_P(field->DEF,key,field->data); break;
          case 'S': key = LcdEditStringSel_P(field->DEF,field->length,key,field->data); break;
        }
      }
      break;
  }
  return key;
}

void LcdFieldInit(struct LcdField *field,boolean init) {
  field->x = 0;
  field->length = LcdFieldLength(field->DEF);
  if (pgm_read_byte(field->DEF) == 'o') {
    field->status = LCDEDIT_NONE;
    if (init) memcpy_P(field->data,field->DEF + 4,field->length);
  } else {
    field->status = LCDEDIT_INIT;
    if (init) {
      switch (pgm_read_byte(field->DEF)) {
        case 'b': case 'c': case 'f': case 'i': case 's':
          memcpy_P(field->data,field->DEF + (field->length << 1) + 1,field->length);
          break;
        case 'C' : case 'S':
          memcpy_P(field->data,field->DEF + strlen_P(field->DEF) - field->length,field->length);
          break;
        case 'd': case 't':
          memcpy_P(field->data,field->DEF + 1,8);
          break;
        case 'o':
          memcpy_P(field->data,field->DEF + 4,field->length);
          break;
      }
    }
  }
}

uint8_t LcdFieldLength(PGM_VOID_P DEF) {
  switch (pgm_read_byte(DEF)) {
    case 'b': case 'f': case 'i': case 's': return (strlen_P(DEF) - 1) / 3;
    case 'c': case 'C' : return 1;
    case 'd': case 't': return 8;
    case 'o': return awlstring_stob_P(DEF + 1,3);
    case 'S': return (strlen_P(DEF) - 4) / (awlstring_stob_P(DEF + 1,3) + 1);
  }
}
