/*
  LcdFields.h
  Process Fields.
  Maps Field's to their data ranges, navigates between fields data.

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LcdFields_h
#define LcdFields_h

#include <Arduino.h>
#include "LcdEdit.h"
#include "LcdField.h"
#include "LcdKey.h"

struct LcdFields {
  PGM_VOID_P        DEF;
  uint8_t           length;
  char              *data;
  struct LcdField   field;
  uint8_t           id;
  enum LCDEDIT      status;
  uint8_t           cursor;
};

/*
  Edit Fields.

  parameter :
  key       key to process
  fields    fields data
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdFieldsEdit(enum LCDKEY key,struct LcdFields *fields);

/*
  Initialise Fields.

  parameter :
  fields    fields data
  init      initialise data
*/
void LcdFieldsInit(struct LcdFields *fields,boolean init);

/*
  Get fields length.

  parameter :
  DEF       pointer to fields definition
  return    fields length
*/
uint8_t LcdFieldsLength(PGM_VOID_P DEF);

#endif

#ifdef __cplusplus
}
#endif
