/*
  LcdKey.h

  LcdKey driver (eg. GHI electronics LCD LcdKey shield v2.0)

  Key discrimination based on resistor network / analog input.
  Scale factors :
  - right : 0
  - up : 330/2330 = 0.14
  - down : 950/2950 = 0.32
  - left : 1950/3950 = 0.49
  - select : 5250/7250 = 0.72
  - none : 1
  Unscaled key switch levels are placed in between these values and multiplied
  by 100.

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LcdKey_h
#define LcdKey_h

#include <Arduino.h>

enum LCDKEY {LCDKEY_NONE,LCDKEY_SELECT,LCDKEY_LEFT,LCDKEY_DOWN,LCDKEY_UP,LCDKEY_RIGHT,LCDKEY_ESC,LCDKEY_EXIT};

/*
  Check
  Check which key is active.

  parameter :
  return    active key
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdKeyCheck();

/*
  Init
  Calibrate analog input.
  remarks
  - no key should be pressed during calibration

  parameter :
  pin         keypad analog input pin
  debounce    button debounce time (ms)
  delay       auto repeat delay time (ms)
  repeat      auto repeat frequency (chars/s)
  exit        inactivity time out (s)
*/
void LcdKeyInit(uint8_t pin,uint16_t debounce,uint16_t delay,uint8_t repeat,uint8_t exit);

/*
  Press
  Check whether a new key is pressed.

  parameter :
  return    key pressed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdKeyPress();

#endif

#ifdef __cplusplus
}
#endif
