/*
  LcdMenu.h
  Process menu.

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LcdMenu_h
#define LcdMenu_h

#include <Arduino.h>
#include "LcdKey.h"
#include "LcdFields.h"

struct LcdMenu {
  int8_t              left;
  int8_t              up;
  int8_t              down;
  int8_t              right;
  PGM_VOID_P          DEF;
  struct LcdFields    fields;
  enum LCDEDIT        status;
  uint8_t             cursor;
  uint8_t             pptr;
  uint8_t             dptr;
};

/*
  Edit menu.
  Remark :
  Placeholder/dummy/convenience function.

  parameters :
  key       key to process
  menu      menu data
  return    key, if not consumed or if menu changed
            LCDKEY_NONE otherwise
*/
enum LCDKEY LcdMenuEdit(enum LCDKEY key,struct LcdMenu *menu);

/*
  Get menu data for display.

  parameters :
  init    initialise pointers
  menu    menu data
*/
char LcdMenuGet(struct LcdMenu *menu,boolean init);

/*
  Initialise menu.

  parameters :
  init    initialise data
  menu    menu data
*/
void LcdMenuInit(struct LcdMenu *menu,boolean init);

#endif

#ifdef __cplusplus
}
#endif
